.model medium,basic                 

.386

.data

.code

public AraBlink
public AraBox3D, AraBoxF, AraBoxT,
public AraCopyBuffer
public AraPalGet, AraPalSet
public AraPoint, AraPSet
public AraPut

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;AraBlink: Turns blinking colours on or off when in textmode.               
;Stack: 06 - Blink Status
;       04 - Basic Return Segment
;       02 - Basic Return Offset
;       00 - BP
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

AraBlink proc
    push bp
    mov bp,sp

    mov ax, 1003h
    mov bx, [bp+06] 
    int 10h

    pop bp
    retf 02
AraBlink endp

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;AraBox3D: Draws a 3D-like box in specified colours from X1,Y1 to X2,Y2.
;Stack: 28 - Buffer Segment
;       26 - Buffer Offset
;       24 - X1
;       22 - Y1
;       20 - X2                   
;       18 - Y2
;       16 - Colour Brightest
;       14 - Colour Bright
;       12 - Colour Normal
;       10 - Colour Dark
;       08 - Colour Darkest
;       06 - Basic Return Segment
;       04 - Basic Return Offset
;       02 - BP
;       00 - X length
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

AraBox3D proc
    push bp                                 ;Save BP.
    mov bp,sp                               ;Get SP in BP.

    mov es, [bp+26]                         ;Get buffer segment

    mov cx, [bp+22]                         ;Subtract X1 from X2 to get the
    mov dx, [bp+18]                         ;   distance across the box and
    sub dx, cx                              ;   store the result on the stack
    push dx                                 ;   at bp.
    mov bp,sp                               ;Get SP in BP again.

    mov cx, [bp+22]                         ;CX holds the current Y position,
    mov dx, [bp+24]                         ;   and DX holds the current X.

    xor bh, bh                              ;Set BH to 0 and set BL to the
    mov bl, [bp+22]                         ;   YPos. Shift BX left by 6
    shl bx, 6                               ;   (to multiply it by 64). Add
    add bh, [bp+22]                         ;   the YPos into BH so BX adds it
    add bx, [bp+24]                         ;   as YPos*256 (so BX=YPos*320).
    add bx, [bp+26]                         ;Add XPos and the buffer offset.

    mov al, [bp+16]                         ;Set AL to the brightest colour.

    mov es:[bx], al                         ;Draw the top left pixel and then
    inc bx                                  ;   move over a pixel in the 
    inc dx                                  ;   screen pointer and X counter.

    mov al, [bp+14]                         ;Set AL to the bright colour.

AraBox3D_TopLineNextX:                      ;Draw the next pixel of the top
                                            ;   line. Plot the pixel then
    mov es:[bx], al                         ;   move over a pixel in the
    inc bx                                  ;   screen pointer and X position
    inc dx                                  ;   counter.

    cmp dx, [bp+20]                         ;If we're not at the end, carry
    jl AraBox3D_TopLineNextX                ;   on.

    mov al, [bp+12]                         ;Get the normal colour in al.

    mov es:[bx], al                         ;Plot the top right pixel.

    add bx, 320                             ;Move down a line in pointer and
    sub bx, [bp]                            ;   Y counter.
    sub dx, [bp]
    inc cx

AraBox3D_MainNextY:                         ;Draw a line on the middle bit.

    mov al, [bp+14]                         ;Draw the left edge piece first
    mov es:[bx], al                         ;   in bright colour.
    inc bx
    inc dx

AraBox3D_MainNextX:                         ;Main loopy/labely type thing.

    mov al, [bp+12]                         ;Get the normal colour in al.
    mov es:[bx], al                         ;Plot a pixel and move over one
    inc bx                                  ;   pixel in the screen pointer
    inc dx

    cmp dx, [bp+20]                         ;If we are not off the edge of the
    jl AraBox3D_MainNextX                   ;   screen go and do it again.

    mov al, [bp+10]                         ;Draw the right edge piece now, in
    mov es:[bx], al                         ;   dark colour.
    inc bx
    inc dx

    add bx, 320                             ;Move down a line.
    sub bx, [bp]
    dec bx
    sub dx, [bp]
    dec dx
    inc cx

    cmp cx, [bp+18]                         ;If we are still on screen, go
    jl AraBox3D_MainNextY                   ;   back for more abuse.

    mov es:[bx], al                         ;Plot the bottom left pixel and
    inc bx                                  ;   move over.
    inc dx

AraBox3D_BottomNextX:                       ;Draw a pixel in the bottom line.

    mov es:[bx], al                         ;Plot the pixel and move over.
    inc bx
    inc dx

    cmp dx, [bp+20]                         ;If we're not off the edge draw
    jl AraBox3D_BottomNextX                 ;   another.

    mov al, [bp+08]                         ;Plot the bottom right pixel,
    mov es:[bx], al

    pop dx                                  ;Restore DX.
    pop bp                                  ;Restore BP.
    retf 14                                 ;Back to basic.
AraBox3D endp

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;AraBoxF: Draws a filled box in colour Colour from X1,Y1 to X2,Y2.
;Stack: 20 - Buffer Segment
;       18 - Buffer Offset
;       16 - X1
;       14 - Y1
;       12 - X2                   
;       10 - Y2
;       08 - Colour
;       06 - Basic Return Segment
;       04 - Basic Return Offset
;       02 - BP
;       00 - X length
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

AraBoxF proc
    push bp                                 ;Save BP.
    mov bp,sp                               ;Get SP in BP.

    mov es, [bp+18]                         ;Get buffer segment

    mov cx, [bp+14]                         ;Subtract X1 from X2 to get the
    mov dx, [bp+10]                         ;   distance across the box and
    sub dx, cx                              ;   store the result on the stack
    push dx                                 ;   at bp.
    mov bp,sp                               ;Get SP in BP again.

    mov cx, [bp+14]                         ;CX holds the current Y position,
    mov dx, [bp+16]                         ;   and DX holds the current X.

    xor bh, bh                              ;Set BH to 0 and set BL to the
    mov bl, [bp+14]                         ;   YPos. Shift BX left by 6
    shl bx, 6                               ;   (to multiply it by 64). Add
    add bh, [bp+14]                         ;   the YPos into BH so BX adds it
    add bx, [bp+16]                         ;   as YPos*256 (so BX=YPos*320).
    add bx, [bp+18]                         ;Add XPos and the buffer offset.

    mov al, [bp+08]                         ;Set AL to the pixels colour.

AraBoxF_NextX:                              ;Main loopy/labely type thing.

    mov es:[bx], al                         ;Plot a pixel and move over one
    inc bx                                  ;   pixel in the screen pointer
    inc dx                                  ;   and X position counter.

    cmp dx, [bp+12]                         ;If we are not off the edge of the
    jle AraBoxF_NextX                       ;   screen go and do it again.

    add bx, 320                             ;Move down a line and back to the
    sub bx, [bp]                            ;   left edge of the box in the
    dec bx                                  ;   screen pointer and the X and
    sub dx, [bp]                            ;   Y position counters.
    dec dx
    inc cx

    cmp cx, [bp+10]                         ;Check if we are off the bottom,
    jle AraBoxF_NextX                       ;   if not then loop again.

    pop dx                                  ;Restore DX.
    pop bp                                  ;Restore BP.
    retf 14                                 ;Back to basic.
AraBoxF endp

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;AraBoxT: Draws a translucent box in a 16-colour-range starting from Colour
;         from X1,Y1 to X2,Y2.
;Stack: 26 - Buffer Segment
;       24 - Buffer Offset
;       22 - X1
;       20 - Y1
;       18 - X2                   
;       16 - Y2
;       14 - Colour
;       12 - Pal Segment
;       10 - Pal Offset
;       08 - Basic Return Segment
;       06 - Basic Return Offset
;       04 - BP
;       02 - DS
;       00 - X length
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

AraBoxT proc
    push bp                                 ;Save BP.
    mov bp,sp                               ;Get SP in BP.

    mov es, [bp+24]                         ;Get buffer segment
    mov ds, [bp+12]

    mov cx, [bp+20]                         ;Subtract X1 from X2 to get the
    mov dx, [bp+16]                         ;   distance across the box and
    sub dx, cx                              ;   store the result on the stack
    push dx                                 ;   at bp.
    mov bp,sp                               ;Get SP in BP again.

    mov cx, [bp+20]                         ;CX holds the current Y position,
    mov dx, [bp+22]                         ;   and DX holds the current X.

    xor bh, bh                              ;Set BH to 0 and set BL to the
    mov bl, [bp+20]                         ;   YPos. Shift BX left by 6
    shl bx, 6                               ;   (to multiply it by 64). Add
    add bh, [bp+20]                         ;   the YPos into BH so BX adds it
    add bx, [bp+22]                         ;   as YPos*256 (so BX=YPos*320).
    add bx, [bp+24]                         ;Add XPos and the buffer offset.

AraBoxT_NextX:                              ;Main loopy/labely type thing.

    push cx                                 ;Save cx and dx on the stack (we
    push dx                                 ;   need them preserved).
    mov bp,sp                               ;Re-get the stack pointer.

    mov cl, es:[bx]
    mov si, [bp+10]
    mov al, 3
    mul cl
    add si, ax
    mov al, ds:[si]
    add al, ds:[si+1]
    add al, ds:[si+2]
    mov cl, al
    mov al, 7
    mul cl
    add al, [bp+18]

    pop dx                                  ;Restore cx and dx again and re-get
    pop cx                                  ;   the stack pointer (again).
    mov bp,sp

    mov es:[bx], al                         ;Plot a pixel and move over one
    inc bx                                  ;   pixel in the screen pointer
    inc dx                                  ;   and X position counter.

    cmp dx, [bp+18]                         ;If we are not off the edge of the
    jle AraBoxT_NextX                       ;   screen go and do it again.

    add bx, 320                             ;Move down a line and back to the
    sub bx, [bp]                            ;   left edge of the box in the
    dec bx                                  ;   screen pointer and the X and
    sub dx, [bp]                            ;   Y position counters.
    dec dx
    inc cx

    cmp cx, [bp+16]                         ;Check if we are off the bottom,
    jle AraBoxT_NextX                       ;   if not then loop again.

    pop dx                                  ;Restore DX.
    pop bp                                  ;Restore BP.
    retf 14                                 ;Back to basic.
AraBoxT endp

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;AraCopyBuffer: Copies 64000 bytes from memory to somewhere else in memory.
;Stack: 14 - Source Segment
;       12 - Source Offset
;       10 - Destination Segment
;       08 - Destination Offset
;       06 - Basic Return Segment
;       04 - Basic Return Offset
;       02 - DS
;       00 - BP
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
AraCopyBuffer proc
    push ds                                 ;Save DS.
    push bp                                 ;Save BP.
    mov bp, sp                              ;Get SP in BP.
                                
    mov ds, [bp+14]                         ;Set DS and SI to the source segment
    mov si, [bp+12]                         ;   and source offset, and do the same
    mov es, [bp+10]                         ;   for ES and DI and the destination
    mov di, [bp+8]                          ;   segment and offset.

    mov cx, 32000                           ;Now move all 64kb from source to
    rep movsw                               ;   destination.

    pop bp                                  ;Restore BP again.
    pop ds                                  ;Restore DS again.
    retf 8                                  ;Return to QB again.
AraCopyBuffer endp

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;AraPalGet: Gets the palette into a string.
;Stack: 08 - Pal Segment
;       06 - Pal Offset
;       04 - Basic Return Segment
;       02 - Basic Return Offset
;       00 - BP
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

AraPalGet proc
    push bp

    mov bp,sp

    mov es, [bp+08]

    xor ax, ax

    mov dx, 3c7h
    out dx, al

    mov di, [bp+06]
    mov cx, 768
    mov dx, 03c9h

AraPalGet_Loop:
    in al, dx
    stosb

    dec cx
    jnz AraPalGet_Loop

    pop bp
    retf 4
AraPalGet endp

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;AraPalSet: Sets the palette from a string.
;Stack: 10 - Pal Segment
;       08 - Pal Offset
;       06 - Basic Return Segment
;       04 - Basic Return Offset
;       02 - BP
;       00 - DS
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

AraPalSet proc
    push ds
    push bp

    mov bp,sp

    mov ds, [bp+10]

    xor ax, ax

    mov dx, 3c8h
    out dx, al

    mov si, [bp+08]
    mov cx, 768
    mov dx, 03c9h

AraPalSet_Loop:
    lodsb
    out dx, al
    dec cx
    jnz AraPalSet_Loop

    pop bp
    pop ds
    retf 4
AraPalSet endp

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;AraPoint: Gets the colour of a pixel.
;Stack: 12 - Buffer Segment
;       10 - Buffer Offset
;       08 - X
;       06 - Y          
;       04 - Basic Return Segment
;       02 - Basic Return Offset
;       00 - BP
;Returns: pixel colour.
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

AraPoint proc
    push bp                                 ;Save BP.
    mov bp, sp                              ;Get SP in BP.

    mov es, [bp+12]                         ;Get buffer segment in ES.

    xor bh, bh                              ;Set BH to 0 and set BL to the
    mov bl, [bp+06]                         ;   YPos. Shift BX left by 6
    shl bx, 6                               ;   (to multiply it by 64). Add
    add bh, [bp+06]                         ;   the YPos into BH so BX adds it
    add bx, [bp+08]                         ;   as YPos*256 (so BX=YPos*320).
    add bx, [bp+10]                         ;Add XPos and the buffer offset.

    mov al, es:[bx]                         ;Get the pixel colour.

    pop bp                                  ;Restore BP.
    retf 8                                  ;Return to QB.
AraPoint endp

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;AraPSet: Puts a pixel
;Stack: 14 - Buffer Segment
;       12 - Buffer Offset
;       10 - X
;       08 - Y          
;       06 - Colour    
;       04 - Basic Return Segment
;       02 - Basic Return Offset
;       00 - BP
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

AraPSet proc
    push bp                                 ;Save BP.
    mov bp, sp                              ;Get SP in BP.

    mov es, [bp+14]                         ;Get buffer segment in ES.

    xor bh, bh                              ;Set BH to 0 and set BL to the
    mov bl, [bp+08]                         ;   YPos. Shift BX left by 6
    shl bx, 6                               ;   (to multiply it by 64). Add
    add bh, [bp+08]                         ;   the YPos into BH so BX adds it
    add bx, [bp+10]                         ;   as YPos*256 (so BX=YPos*320).
    add bx, [bp+12]                         ;Add XPos and the buffer offset.

    mov al, [bp+06]                         ;Set AL to the pixels colour.

    mov es:[bx], al                         ;Plot the pixel.

    pop bp                                  ;Restore BP.
    retf 10                                 ;Return to QB.
AraPSet endp

;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;AraPut: Puts a sprite transparently.
;Stack: 24 - Buffer Seg
;       22 - Buffer Off
;       20 - Sprite Seg
;       18 - Sprite Off
;       16 - XPos
;       14 - YPos
;       12 - Basic Return Segment
;       10 - Basic Return Offset
;       08 - DS
;       06 - BP
;       04 - Height
;       02 - Width
;       00 - Width in Screen
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

AraPut proc
    push ds                                 ;Save DS.
    push bp                                 ;Save BP.
    push eax                                ;Make space for variables on the 
    push ax                                 ;   stack.

    mov bp, sp                              ;Get SP in BP.

    mov si, [bp+18]                         ;Get sprite offset.
    mov ds, [bp+20]                         ;Get sprite segment.

    mov ecx, [si]                           ;Get sprite width in bits, shift
    mov es, [bp+24]                         ;Get screen segment.
    shr cx, 3                               ;   it right by 3 to convert it to
    mov [bp+2], ecx                         ;   bytes and store it as Width
    mov [bp], cx                            ;   and as Width in Screen.

    mov ax, [bp+16]                         ;Set sprite offset to first pixel,
    add si, 4                               ;if the XPos is off the right edge
    cmp ax, 319                             ;   of the screen then end the
    jg AraPut_End                           ;   proc.
    neg ax                                  ;If the sprite is completely off 
    cmp ax, cx                              ;   the left edge of the screen
    jns AraPut_End                          ;   then end the proc.
    
    mov ax, [bp+14]                         ;If the YPos is off the bottom
    cmp ax, 199                             ;   of the screen then end the 
    jg AraPut_End                           ;   proc.
    neg ax                                  ;If the sprite is completely off 
    cmp ax, [bp+4]                          ;   the top edge of the screen
    jns AraPut_End                          ;   then end the proc.
    
    mov ax, [bp+16]                         ;Get the XPos in AX and if it is
    cmp ax, 0                               ;   less then 0 then skip doing
    jns AraPut_SkipAddX1                    ;   this.
    mov bx, ax                              ;Copy XPos into BX and negate AX
    neg ax                                  ;   (which is XPos).
    add bx, [bp+2]                          ;Add Width to BX so BX=Width+Xpos.
    add si, ax                              ;Move forward in the sprite.
    mov [bp], bx                            ;Set the WidthInScreen to Width+X
    add [bp+16], ax                         ;   and move forward in screen.
    jmp AraPut_DoneX                        ;Now skip this next bit.

AraPut_SkipAddX1:

    add ax, [bp+2]                          ;Add Width to AX and if it is >319
    cmp ax, 319                             ;   then skip doing all of this
    js AraPut_DoneX                         ;   bit.
    mov ax, 320                             ;Subtract the XPos from 320 and
    sub ax, [bp+16]                         ;   set the Width in Screen to be
    mov [bp], ax                            ;   it.

AraPut_DoneX:

    mov bx, [bp+14]                         ;Get the YPos in AX and if it is
    cmp bx, 0                               ;   less than 0 then skip doing
    jns AraPut_SkipAddY1                    ;   this.

    neg bx                                  ;Negate BX (now -YPos) and
    mov ax, bx                              ;   and multiply it by itself
    mul bx                                  ;   with the result in AX.
    sub [bp+4], bx                          ;Subtract height from BX and then
    add si, ax                              ;   add AX to SI.
    mov word ptr [bp+14], 0                 ;
    jmp AraPut_DoneY                        ;Done the Y now.

AraPut_SkipAddY1:

    add bx, [bp+4]                          ;Add the height to BX and if it is
    cmp bx, 199                             ;   <=199 then skip all of this
    jng AraPut_DoneY                        ;   bit.
    mov ax, 200                             ;Set AX to 200 and subtract the 
    sub ax, [bp+14]                         ;   YPos from it. Set the height
    mov [bp+4], ax                          ;   to that.

AraPut_DoneY:

    xor bh, bh                              ;Set BH to 0 and set BL to the
    mov bl, [bp+14]                         ;   YPos. Shift BX left by 6
    shl bx, 6                               ;   (to multiply it by 64). Add
    add bh, [bp+14]                         ;   the YPos into BH so BX adds it
    add bx, [bp+16]                         ;   as YPos*256 (so BX=YPos*320).
    add bx, [bp+22]                         ;Add XPos and the buffer offset.

    mov ax, [bp+4]                          ;Add the height to AX and set DI
    mov di, [bp]                            ;   to the Width in Screen.

AraPut_Loop:

    mov cx, di                              ;Set CX to DI

AraPut_PixelLoop:

    mov dl, [si]                            ;Get the current pixel colour from
    inc si                                  ;   the sprite and increment the
    cmp dl, 0                               ;   offset in the sprite. If the
    jz AraPut_SkipPixel                     ;   colour is 0 then skip this
    mov es:[bx], dl                         ;   otherwise plot the pixel.

AraPut_SkipPixel:

    inc bx                                  ;Increment the offset on the
    dec cx                                  ;   decrement CX. If CX is <=0
    jg AraPut_PixelLoop                     ;   then loop again.

    add bx, 320                             ;Add 320 to screen offset and add
    add si, [bp+2]                          ;   sprite width to sprite offset.
    sub bx, di                              ;Subtract DI from BX and DI from
    sub si, di                              ;   SI.

    dec ax                                  ;Decrement AX and if it is <=0
    jg AraPut_Loop                          ;   then loop again.


AraPut_End:

    pop ax                                  ;Remove variables space from stack
    pop eax                                 ;   here and then restore the BP
    pop bp                                  ;   register and restore the DS
    pop ds                                  ;   register.
    retf 12                                  ;Return from proc.
AraPut endp

end

